from .base import BaseView, Resource
from .proxy import ConfProxyView, ProxyView, TransparentProxyView
from .resource import CatalogResource, Restful
from .rpc import JSONRPCView
from .template import RenderTemplate
